// --- [ eeprom.c ] -----------------------------------------------------------

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include "global.h"
#include "rs232_int.h"
#include "xbee_tx_a.h"

void print_adr(uint8_t start);

// ----------------------------------------------------------------------------
// Schreibt genau 1 Byte an die angegebene Adresse im Eeprom

void ee_write_byte(uint8_t eeprom_adr, uint8_t data)
{
	while (EECR & (1<<EEPE));
/* ATMEGA88
	EEARH = (uint8_t) (eeprom_adr >> 8);	// Adresse schreiben
*/	EEARL  = (uint8_t) eeprom_adr;

	EEDR  = data;									// Daten schreiben

	cli();
	EECR |= (1<<EEMPE);
	EECR |= (1<<EEPE);
	sei();
}

// ----------------------------------------------------------------------------
// Liest genau 1 Byte an der angegebenen Adresse aus dem Eeprom
uint8_t ee_read_byte(uint8_t eeprom_adr)
{
	while (EECR & (1<<EEPE));
/* ATMEGA88
	EEARH = (uint8_t) (eeprom_adr >> 8);	// Adresse schreiben
*/	EEARL = (uint8_t) eeprom_adr;
	EECR |= (1<<EERE);							// Eeprom lesen
	return(EEDR);
}

// -----------------------------------------------------------------------------
// Gibt einen Speicherauszug des Eeproms aus
// in Zeile 1 werden die Variablen Autostart (0/255) und das Messintervall ausgegeben
// ab Zeile 2 folgen die Sensoren
void ee_print_eeprom(void)
{
uint8_t i, j;

put_c(13);											// Leerzeile ausgeben
put_c(13);
for (i = 8; i < EEPROM_SIZE; i += 8)		// Speicherauszug Eeprom drucken
	{
	for(j = 0; j < 8; j++)						// jeweils 8 Byte pro Zeile = 1 Sensor
		{
		put_s(utoa(ee_read_byte(i + j), msg, 16));
		put_c(' ');
		}
		put_c(13);
	}
}



// -----------------------------------------------------------------------------
// Sendet einen Speicherauszug des Eeproms via XBee an den Koordinator
// und zwar jeweils 1 Zeile bestehend aus 8 Byte = Adressdaten fr 1 Sensor
// ab Zeile 2 folgen die Sensoren
void ee_send_eeprom(void)
{
uint8_t j;
for (j = 8; j < EEPROM_SIZE; j += 8) print_adr(j);
}

// ------------------------------------------------------------------------------
//
void print_adr(uint8_t start)
{
uint8_t i;
uint8_t data[26];
//char cr[2] = {13, 0};
data[0] = 0;

for(i = start; i < start + 8; i++)
	{
	strcat((char *)data, utoa(ee_read_byte(i), msg, 16));
	strcat((char *)data, delimiter);
	}
strcat((char *)data, cr);
FRM10_send_data(data, strlen((char*) data), 0);
//while (rs232_tx_busy());
	
}

// --- [ eof ] ----------------------------------------------------------------