// --- [ main.c ] -------------------------------------------------------------
//       tab = 3
//
//      Temperaturmessung mit 1-Wire-Bus / Sensoren DS18B20 (ROM-SEARCH-free)
//
//      Die Sensoren sind an einem sleeping Enddevice angeschlossen
//
//      Die ROM-Adressen der beteiligten Sensoren sind im EEPROM angelernt.
//      Via RS232 knnen Kommandos angenommen werden:
//      - 0x30 Anzeigen des ROM_Codes des einzigen angeschlossenen Sensors
//      - 0x31 Anlernen eines Sensors (ROM-Code einlesen und im EEPROM speichern)
//      - 0x40 Alle ROM_Codes im EEPROM lschen
//      - 0x50 Anzeigen aller ROM_Codes im EEprom
//
//     Die Daten werden via XBee (API-Modus) verschickt.
//     Zum Debuggen kann die serielle Schnittstelle 'abgehrt' werden.
//      
// ----------------------------------------------------------------------------

#define EXTERN

#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <stdio.h>

#include "global.h"
#include "eeprom.h"
#include "owi_lib.h"

#include "rs232_int.h"
#include "xbee_rx_a.h"
#include "xbee_tx_a.h"

// --- lokale Variablen ------------------------------------------------------

// hier knnen die Adressen von 5 Stationen eingeben werden.
// die beiden ersten Eintrge fr Coordinator und Broadcast sollten
// unverndert bleiben.

// Die Adressen 2-4 mssen angepasst werden !!!!!!!!!!!!

uint8_t xbee_target_adr[5][10] = {
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFE},
{0x00, 0x13, 0xA2, 0x00, 0x40, 0x33, 0x7B, 0xB7, 0xFF, 0xFE},	// <--- 2
{0x00, 0x13, 0xA2, 0x00, 0x40, 0x32, 0x09, 0x3B, 0xFF, 0xFE},	// <--- 3
{0x00, 0x13, 0xA2, 0x00, 0x40, 0x32, 0x09, 0x8B, 0xFF, 0xFE}	// <--- 4
};

// --- lokale Funktionen ------------------------------------------------------

// ----------------------------------------------------------------------------
int main(void)
{
uint8_t i, rx_received;
char cmd[32]; 

cr[0] = 13;											// RETURN in Variable
cr[1] = 0;
delimiter[0] = DELIMITER;						// Trennzeichen in Variable
delimiter[1] = 0;

ONE_WIRE_DDR  = (1<<ONE_WIRE_POWER_PIN | 1<<ONE_WIRE_PORT_PIN);
ONE_WIRE_PORT = (1<<ONE_WIRE_POWER_PIN | 1<<ONE_WIRE_PORT_PIN);

// ATMEGA										// Strom sparen, Timer1/2 und ADC abschalten, nur MEGAx8
PRR = (1<<PRTWI | 1<<PRTIM2 | 1<<PRTIM1 | 0<<PRTIM0 | 1<<PRSPI | 0<<PRUSART0 | 1<<PRADC);

// --- Clock 
CLKPR = (1<<CLKPCE);								// Clockprescale = 1
CLKPR = 0; 

XBEE_rx_timer0_init();							// fr den Empfnger notwendig

// --- die Serielle Schnittstelle aktivieren
rs232_init();
asm volatile ("sei");							// fr Timer und RX erforderlich

// --- im Eeprom gespeicherte Daten auslesen: Sensorcodes, Anzahl Sensoren
OW_read_eeprom();					

ee_print_eeprom();								// einen Speicherauszug des Eeproms
put_s("Start");
put_c(13);

// der Ausgang SLEEP des XBee wird invertiert und mit INT0 verbunden
// ist SLEEP high (XBee aktiv), dann ist INT0 low
// ist SLEEP low  (XBee sleeping), dann ist INT0 high
// der bergang von high auf low lst den Interrupt aus.
// wenn der Pin INT0 low ist, dann warten auf Kommandos zur Konfiguration
//
// ein Transistor o.. muss den Status des Pin SLEEP der XBee invertieren,
// damit am ATMega an Pin INT0 der richtige Pegel fr ein wakeup vorliegt !!!!!
//
// die Ausgabe erfolgt hier ber die Serielle Schnittstelle !!!

while (!(PIND & (1<<PD2)))						// "Vorprogramm" -> Konfiguration, solange wie INT0 low/ SLEEP high ist
	{
	if (owi_cmd)									// es wurde 1 Byte via rs232 empfangen
		{

		if (owi_cmd == 13) owi_cmd = 0; 		// ein Return ausblenden	
														// whrend in den nachfolgenden Zeilen ein Echo der Terminaleingabe
														// zurckgeschrieben wird, kommen unter Umstnden noch weitere
														// Zeichen ber die serielle Schnittstelle (wie RETURN) - und berschreiben rx_in.
		rx_received = owi_cmd;					// Darum wird rx_in umkopiert auf rx_received.
														// 
		put_s("rx: 0x");							// als Rckkoppelung ein Echo der Eingabe ausgeben 						
		put_s(utoa(rx_received, msg, 16)); 
		put_c(' ');

		if (rx_received == 0x30)				// Sensor-Rom-Code auslesen, nur den Code ausgeben
			{
			OW_read_rom_code();					// 
			}	
			
		else if (rx_received == 0x31)			// Sensor anlernen, Code speichern
			{
			if (OW_read_rom_code())				// wenn der ROM-Code fehlerfrei ist
				{
				if (!(OW_rom_code_exists()))	// und noch nicht vorhanden ist
					{
					if ((i = OW_eeprom_free()))// und noch Platz im Eeprom ist
						{
						EEPROM_write_rom_code(i);// dann im EEPROM ablegen
						sensor_cnt++;				// Anzahl der Sensoren mitzhlen
						put_c('#');
						put_s(utoa(sensor_cnt, msg, 10)); // und ausgeben 
						}
					}
				}
			}

		else if (rx_received == 0x40)			// EEprom lschen
			{											// alles ab Adr 0 mit 0xFF berschreiben
			for (i = 0; i < EEPROM_SIZE; i++) ee_write_byte(i, 0xFF);
			sensor_cnt = 0;
			}
		
		else if (rx_received == 0x50)			// alle ROM-Codes via RS232 ausgeben
			{
			ee_print_eeprom();					// um die aktuellen Eeprom-Wert anzuzeigen
			}
		
		EEARH = 0;									// gibts nicht beim ATTINY/ATMEGA48
		EEARL = 0;									// eepromadresse auf 0 stellen
		put_c(13);									// Zeilenvorschub
		owi_cmd = 0;
		}
	}													// while (PIND & (1<<PD2))

/*
strcpy(cmd, "SO");								// AT-Command versenden
cmd[2] = 2 + 4;
flag = 0;
FRM08_send_lokal_at_cmd((uint8_t*)cmd, 3);
while (!(flag));
*/

/*
strcpy(cmd, "SP");								// Sleep 3 * 2560ms
cmd[2] = 2;
cmd[3] = 0xFF;
flag = 0;
FRM08_send_lokal_at_cmd((uint8_t *)cmd, 4);
while (!(flag));
*/

/*
strcpy(cmd, "ST");
cmd[2] = 9;
cmd[3] = 0x0A;					// default = 5 Sekunden
flag = 0;
FRM08_send_lokal_at_cmd((uint8_t *)cmd, 4);
while (!(flag));
*/

/*
strcpy(cmd, "PO");
cmd[2] = 4;
cmd[3] = 0;
flag = 0;
FRM08_send_lokal_at_cmd((uint8_t*)cmd, 4);
while (!(flag));
*/


while(1)												// hier beginnt die Hauptschleife
	{

	if (!(PIND & (1<<PD2)))						// wenn der Pin noch low (XBee aktiv),
		{												// dann Sleep erzwingen
		strcpy(cmd, "SI");
		flag = 0;
		FRM08_send_lokal_at_cmd((uint8_t*)cmd, 2);
		while ((!(flag)) && (!(PIND & (1<<PD2)))); 
		}
														// XBee-Sleep_Pin -> low = SLEEP
														// der Pin wechselt von low nach high beim Aufwachen
														// der AVR bentigt genau den umgekehrten Wechsel von
														// high nach low fr den Low Level Interrupt
														// Daher muss der Pegel invertiert werden !!!!

	while (!(PIND & (1<<PD2)));				// warten, bis der Pin auf high geht 
														// d.h. der XBee-Sleep-Pin auf low
														// Es wird sofort wieder ein low level interupt ausgelst
														// wenn der Pin noch low ist !!

														// --- INT0 	
	EICRA = (0<<ISC01 |  0<<ISC00);			// low level Interrupt
	EIMSK = (1<<INT0);							// INTO aktivieren
														// --- Sleep
	
	SMCR = (0<<SM2 | 1<<SM1 | 0<<SM0 |1<<SE); // Powerdown-Modus, Sleep enable
	asm volatile ("sleep");						// das Sleep wird durch einen INT0_IRQ beendet
														// die INT0-ISR setzt EIMSK zurck
	SMCR = 0;										// Sleep disablen
	
	OW_convert_all();								// die Messungen anstoen, dabei wird 800ms gewartet,	

	cmd[0] = 0;										// den String lschen
	for (i = 0; i < sensor_cnt; i++)			// die Messstellen nacheinander auslesen
		{												// dazu die (Sensornummer + 1) bergeben,
		if (OW_read_scratchpad(i + 1))		// wenn Daten ohne CRC-Fehler eingelesen wurden
			{											// den Index um 1 erhhen, da die Bytes 0..7 keine ROM-Daten enthalten
														// das Ergebnis steht in ow_buffer[],
			DS18B20_convert_temperatur();		// von dort holt sich die Funktion die Messwerte und wandelt sie um,
			strcat(cmd, msg);						// den Wert in den String cmd einlesen
			strcat(cmd, delimiter);				// Trennzeichen anhngen
			}
			
		else strcat(cmd, delimiter);			// bei einem Fehler in der Messung nur ein Trennzeichen ausgeben
		}
//	strcat(cmd, cr);								// ein Return als Zeilenvorschub anhngen
														// die Nachricht senden
	flag = 0;										// fr die Abfrage auf flag & (1<<FRM_COMPLETE) vorbereiten
	FRM10_send_data((uint8_t *) cmd, strlen(cmd), 2);
	while ((!(flag)) && (!(PIND & (1<<PD2)))); // warten bis zur Rckmeldung oder bis XBee zum Sleep bergeht


	put_c(13);										// zur Kontrolle die Rckmeldung anzeigen
	put_s(itoa(FRM_get_frame_typ(),msg,16)); // Frame-Typ
	put_c(' ');	
	put_s(itoa(FRM8B_get_delivery_status(),msg,16)); // Fehlernummer
	put_c(13);

	}
}

// ----------------------------------------------------------------------------
// Der Interrupt wird nur bentigt, um aus dem Sleep-Modus wieder aufzuwachen
// Der Interrupt lst solange aus, wie INT0 auf gnd liegt.
// Deswegen muss er sofort deaktiviert werden
ISR(INT0_vect)
{
EIMSK = 0;											// den Interrupt disablen
}


// --- [ eof ] ----------------------------------------------------------------