// --- [ xbee_tx.c ] ---------------------------------------------------------
//
// Ist fr das Senden via Frame0x08, Frame0x10 und Frame0x17 zustndig
//
// Framelnge: 0x08:  7 + Daten
//             0x10: 17 + Daten
//             0x17: 18 + Daten

#define CMD_BUFFER_SIZE       32

#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>

#include "global.h"
#include "xbee_tx_a.h"
#include "rs232_int.h"

// ---------------------------------------------------------------------------
uint8_t cmd_buffer[CMD_BUFFER_SIZE];
uint8_t frame_id = 0;
//uint8_t tx_status[8][8];
//uint8_t tx_status_idx = 0;

uint8_t checksum(void);
void FRM_to_air(uint8_t cnt);

// ---------------------------------------------------------------------------
// baut ein AT-Command auf und schreibt es in den Sendepuffer
void FRM08_send_lokal_at_cmd(uint8_t * cmd, uint8_t cmd_length)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x08;                   // Frame Typ
if (!(frame_id)) frame_id++;              // wenn Frame_ID == 0, dann Frame_ID = 1
cmd_buffer[i++] = frame_id++;
														// das Kommando incl. Parameter anhngen
while (cmd_length--)	cmd_buffer[i++] = cmd[j++];

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = checksum();             // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}

// ----------------------------------------------------------------------------
void FRM10_send_data(uint8_t * data, uint8_t cmd_length, uint8_t target_idx)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x10;                   // Frame Typ
if (!(frame_id)) frame_id++;              // wenn Frame_ID == 0, dann Frame_ID = 1
cmd_buffer[i++] = frame_id++;

for (j = 0; j < 10; j++)                  // 8 Byte Destination_Adr + 2 Byte
	{
	cmd_buffer[i++] = xbee_target_adr[target_idx][j];
	}

cmd_buffer[i++] = 0x0;                    // Broadcast Radius
cmd_buffer[i++] = 0x0;                    // Options

j = 0;
while (cmd_length--)	cmd_buffer[i++] = data[j++]; // Data anhngen

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = checksum();             // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}


// ----------------------------------------------------------------------------
void FRM17_send_remote_at_cmd(uint8_t * cmd, uint8_t cmd_length, uint8_t target_idx)
{
uint8_t i = 0, j = 0;

cmd_buffer[i++] = START_DELIMITER;
cmd_buffer[i++] = 0;
i++;                                      // die Lnge berspringen
cmd_buffer[i++] = 0x17;                   // Frame Typ
if (!(frame_id)) frame_id++;              // wenn Frame_ID == 0, dann Frame_ID = 1
cmd_buffer[i++] = frame_id++;

for (j = 0; j < 10; j++)                  // 8 Byte Destination_Adr + 2 Byte
	{
	cmd_buffer[i++] = xbee_target_adr[target_idx][j];
	}

cmd_buffer[i++] = 0x02;                   // Apply Changes

j = 0;
while (cmd_length--)	cmd_buffer[i++] = cmd[j++]; // Kommando + Parameter anhngen

cmd_buffer[2] = i - 3;                    // die Kommandolnge einfgen
cmd_buffer[i++] = checksum();             // die Prfsumme ins letzte Byte schreiben

FRM_to_air(i);                      		// schreibt das Frame in den Sendepuffer
}


// ----------------------------------------------------------------------------
// addiert alle Bytes von Position 3 bis zum Byte vor der Prfsumme
// und subtrahiert das Ergebnis von 0xFF
uint8_t checksum(void)
{
uint8_t end, j, i = 0;

end = cmd_buffer[2] + (3 - 1);            // die Prfsumme nicht in die Prfsumme einschlieen

for (j = 3; j <= end; j++) i += cmd_buffer[j]; // die Prfsumme zhlt ab Index 3

return(0xFF - i);                         // den Wert von 255 abziehen
}

// ----------------------------------------------------------------------------
// Versendet ein fertig aufbereitetes Frame (das im Array cmd_buffer[] steht).
// es gibt unterschiedliche Methoden zum Senden der Daten
// - direktes Senden des Arrays
//   Vorteil: einfach zu implementieren
//   Nachteil: das Programm hngt lange in einer Schleife
// - das Array in einen Sendepuffer schreiben und via Interrupt die Bytes absenden
//   Vorteil: das Programm kann nach bergabe der Daten in die Hauptschleife zurck
//   Nachteil: bentigt Speicher fr einen Ringpuffer
//
//   in der Datei rs232_int.c/h kann zwischen Interrupt und Polling umgestellt werden
void FRM_to_air(uint8_t cnt)
{
uint8_t i;
for (i = 0; i < cnt; i++) put_c(cmd_buffer[i]);
}

/*
void status_add_transmi(void)
in status_tx
void status_del_tx(void)
*/

// --- [ eof ] ----------------------------------------------------------------
